@@ -1,9 +1,12 @@
+# threshold.type
+# segmented2 differs from segmented in parameterization, it is the model studied in Cheng 2008
+# quadupperhinge and quadhinge differ from upperhinge and hinge in that they have a quadratic term
 expit.2pl=function(x,e,b) sapply(x, function(x) 1/(1+exp(-b*(x-e))))
 sim.chngpt = function (
     mean.model=c("thresholded","thresholdedItxn","quadratic","quadratic2b","cubic2b","exp","flatHyperbolic","z2","z2hinge","z2segmented","z2linear"), 
-    # segmented2 differs from segmented in parameterization, it is the model studied in Cheng 2008
-    # upperhingequad differs from upperhinge in that it has a quadratic term
-    threshold.type=c("NA","step","hinge","segmented","segmented2","stegmented","upperhinge","upperhingequad"),
+    threshold.type=c("NA","step","hinge","segmented","segmented2","stegmented"
+        ,"upperhinge","quadupperhinge","quadhinge","cubicupperhinge","cubichinge"
+        ,"M21","M12","M22","M22c","M31","M13","M33c"),
     b.transition=Inf,
     family=c("binomial","gaussian"), 
     x.distr=c("norm","norm3","norm6","imb","lin","mix","gam","zbinary","gam1","gam2", "fixnorm"), # gam1 is a hack to allow e. be different
@@ -107,21 +110,43 @@ sim.chngpt = function (
         if(is.null(alpha)) alpha=try(chngpt::sim.alphas[[mean.model%.%"_"%.%sub("fix","",x.distr)]][e.%.%"", ifelse(mean.model=="thresholdedItxn",beta.itxn,beta)%.%""], silent=TRUE)
         if(is.null(alpha) | inherits(alpha, "try-error")) stop("alpha not found, please check beta or provide a null") 
         
-        X=cbind(1,     z,        x,   x.gt.e,   if(threshold.type=="segmented2") x.gt.e*x else x.gt.e*(x-e.),     z*x,   z*x.gt.e,   z*x.gt.e*(x-e.), x.lt.e*(x-e.), (x.lt.e*(x-e.))^2)
-        coef.=c(alpha, z=coef.z, x=0, x.gt.e=0, x.hinge=0,                                                        z.x=0, z.x.gt.e=0, z.x.hinge=0,     x.lt.e=0,       x.lt.e.quad=0)
+        X=cbind(1,     z,        x,   x.gt.e,   if(threshold.type=="segmented2") x.gt.e*x else x.gt.e*(x-e.),     z*x,   z*x.gt.e,   z*x.gt.e*(x-e.), x.lt.e*(x-e.), (x.lt.e*(x-e.))^2, (x.gt.e*(x-e.))^2, (x.lt.e*(x-e.))^3, (x.gt.e*(x-e.))^3)
+        coef.=c(intercept=alpha, z=coef.z, x=0, x.gt.e=0, x.hinge=0,                                                        z.x=0, z.x.gt.e=0, z.x.hinge=0,     x.uhinge=0,     x.uhinge.quad=0,   x.hinge.quad=0,     x.uhinge.cubic=0,   x.hinge.cubic=0)
         if (mean.model=="thresholded") { 
             if (threshold.type=="step") {
                 coef.[1:5]=c(alpha, coef.z,          0,    beta,     0) 
             } else if (threshold.type=="hinge") {
                 coef.[1:5]=c(alpha, coef.z,          0,       0,  beta) 
+            } else if (threshold.type=="quadhinge") {
+                coef.[1:5]=c(alpha, coef.z,          0,       0,     0); coef.["x.hinge"]=beta[1];  coef.["x.hinge.quad"]=beta[2]
+            } else if (threshold.type=="cubichinge") {
+                coef.[1:5]=c(alpha, coef.z,          0,       0,     0); coef.["x.hinge"]=beta[1];  coef.["x.hinge.quad"]=beta[2];  coef.["x.hinge.cubic"]=beta[3]
             } else if (threshold.type=="upperhinge") {
-                coef.[1:5]=c(alpha, coef.z,          0,       0,     0); coef.["x.lt.e"]=beta 
-            } else if (threshold.type=="upperhingequad") {
-                coef.[1:5]=c(alpha, coef.z,          0,       0,     0); coef.["x.lt.e"]=beta[1]; coef.["x.lt.e.quad"]=beta[2]
+                coef.[1:5]=c(alpha, coef.z,          0,       0,     0); coef.["x.uhinge"]=beta 
+            } else if (threshold.type=="quadupperhinge") {
+                coef.[1:5]=c(alpha, coef.z,          0,       0,     0); coef.["x.uhinge"]=beta[1]; coef.["x.uhinge.quad"]=beta[2]
+            } else if (threshold.type=="cubicupperhinge") {
+                coef.[1:5]=c(alpha, coef.z,          0,       0,     0); coef.["x.uhinge"]=beta[1]; coef.["x.uhinge.quad"]=beta[2]; coef.["x.uhinge.cubic"]=beta[3]
             } else if (threshold.type=="segmented") {
                 coef.[1:5]=c(alpha, coef.z,  -log(.67),       0,  beta) 
             } else if (threshold.type=="segmented2") {
                 coef.[1:5]=c(alpha, coef.z,  -log(.67),       0,  beta) 
+            } else if (threshold.type=="M12") {
+                coef.[1:5]=c(alpha, coef.z,  0,       0,  0); coef.["x.uhinge"]=beta[1]; coef.["x.hinge"]=beta[2]; coef.["x.hinge.quad"]=beta[3]; 
+            } else if (threshold.type=="M21") {
+                coef.[1:5]=c(alpha, coef.z,  0,       0,  0); coef.["x.uhinge"]=beta[1]; coef.["x.uhinge.quad"]=beta[2]; coef.["x.hinge"]=beta[3];
+            } else if (threshold.type=="M22") {
+                coef.[1:5]=c(alpha, coef.z,  0,       0,  0); coef.["x.uhinge"]=beta[1]; coef.["x.uhinge.quad"]=beta[2]; coef.["x.hinge"]=beta[3]; coef.["x.hinge.quad"]=beta[4]; 
+            } else if (threshold.type=="M22c") {
+                coef.[1:5]=c(alpha, coef.z,  0,       0,  0); coef.["x.uhinge"]=beta[1]; coef.["x.uhinge.quad"]=beta[2]; coef.["x.hinge"]=beta[1]; coef.["x.hinge.quad"]=beta[3]; 
+
+            } else if (threshold.type=="M31") {
+                coef.[1:5]=c(alpha, coef.z,  0,       0,  0); coef.["x.uhinge"]=beta[1]; coef.["x.uhinge.quad"]=beta[2]; coef.["x.uhinge.cubic"]=beta[3]; coef.["x.hinge"]=beta[4];
+            } else if (threshold.type=="M13") {
+                coef.[1:5]=c(alpha, coef.z,  0,       0,  0); coef.["x.hinge"]=beta[1]; coef.["x.hinge.quad"]=beta[2]; coef.["x.hinge.cubic"]=beta[3]; coef.["x.uhinge"]=beta[4];
+            } else if (threshold.type=="M33c") {
+                coef.[1:5]=c(alpha, coef.z,  0,       0,  0); coef.["x.uhinge"]=beta[1]; coef.["x.hinge"]=beta[1]; coef.["x.uhinge.quad"]=beta[2]; coef.["x.hinge.quad"]=beta[2]; coef.["x.uhinge.cubic"]=beta[3]; coef.["x.hinge.cubic"]=beta[4]; 
+
             } else if (threshold.type=="stegmented") {
                 coef.[1:5]=c(2,     coef.z,   log(.67), log(.67), beta) # all effects of x in the same direction, subject to perfect separation, though that does not seem to be the main problem
                 #coef.[1:5]=c(0, coef.z, -log(.67), log(.67), beta) # effects of x and x.gt.e in different direction
@@ -161,7 +186,7 @@ sim.chngpt = function (
     } else if (mean.model=="cubic2b") { 
     # x+x^2+x^3
         X=cbind(1,     z,        x,   x*x,   x*x*x)
-        coef.=c(alpha=-1, z=coef.z, x=-1 , x.quad=beta, x.cube=1)
+        coef.=c(alpha=-1, z=coef.z, x=-1 , x.quad=beta, x.cubic=1)
     
     } else if (mean.model=="exp") { 
         if(x.distr=="norm") {
@@ -267,5 +292,7 @@ sim.chngpt = function (
         dat$sampling.p=rep(1, nrow(dat))
     }
     
+    attr(dat, "coef")=coef.
+    attr(dat, "chngpt")=e.
     dat
 }
